//******************************************************************************
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    AUTHOR SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//******************************************************************************

#define HAL_I2C_MODULE_ENABLED
#define HAL_GPIO_MODULE_ENABLED
#include "mems2_i2c_lib.h"
#include "stm32f1xx_hal.h"


I2C_HandleTypeDef hi2c_LIS35;

//LIS35 hardware initialization for I2C operation
void LIS35_I2C_HwConfig(void)
{
  //GPIO and I2C configuration
  GPIO_InitTypeDef  GPIO_InitStruct;

  if (LIS35_I2C == I2C1) __HAL_RCC_I2C1_CLK_ENABLE(); else
  if (LIS35_I2C == I2C2) __HAL_RCC_I2C2_CLK_ENABLE();

  if (LIS35_I2C_GPIO == GPIOA) __HAL_RCC_GPIOA_CLK_ENABLE(); else
  if (LIS35_I2C_GPIO == GPIOB) __HAL_RCC_GPIOB_CLK_ENABLE(); else
  if (LIS35_I2C_GPIO == GPIOC) __HAL_RCC_GPIOC_CLK_ENABLE(); else
  if (LIS35_I2C_GPIO == GPIOD) __HAL_RCC_GPIOD_CLK_ENABLE(); else
  if (LIS35_I2C_GPIO == GPIOE) __HAL_RCC_GPIOE_CLK_ENABLE();

  //GPIO Config
  GPIO_InitStruct.Pin = LIS35_SCL | LIS35_SDA;      //I2C - SCL & SDA
  GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(LIS35_I2C_GPIO, &GPIO_InitStruct);

  //I2C Config
  hi2c_LIS35.Instance = LIS35_I2C;
  hi2c_LIS35.Init.ClockSpeed = 100000;
  hi2c_LIS35.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c_LIS35.Init.OwnAddress1 = 0;
  hi2c_LIS35.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c_LIS35.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c_LIS35.Init.OwnAddress2 = 0;
  hi2c_LIS35.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c_LIS35.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  HAL_I2C_Init(&hi2c_LIS35);
}


//LIS35 initialization
//Range - select measurement range 2 = +-2g; 9=+-9g
char LIS35_I2C_Init(unsigned char Range)
{
  unsigned char RegVal, LIS35Settings;

  if (Range!=2 && Range!=9) return LIS35_ERROR;                             //Check Range selection

  //reset LIS35 settings
  LIS35_I2C_WriteRegister(LIS35_REG_CR2, LIS35_REG_CR2_BOOT);

  //Write settings - activate all axis
  LIS35Settings = LIS35_REG_CR1_DR_100HZ | LIS35_REG_CR1_XEN | LIS35_REG_CR1_YEN | LIS35_REG_CR1_ZEN | LIS35_REG_CR1_ACTIVE;
  if (Range==9) {LIS35Settings = LIS35Settings | LIS35_REG_CR1_FULL_SCALE;} //Full Scale select
  LIS35_I2C_WriteRegister(LIS35_REG_CR1, LIS35Settings);
  //Read configuration - if OK, LIS35 is up and running
  LIS35_I2C_ReadRegister(LIS35_REG_CR1, &RegVal);
  if (RegVal == LIS35Settings) return LIS35_OK;
  else return LIS35_ERROR;
}


//Reads one register value from LIS35
void LIS35_I2C_ReadRegister(unsigned char addr, unsigned char *v)
{
  HAL_I2C_Mem_Read(&hi2c_LIS35, LIS35_I2C_Addr, addr, 1, v, 1, 50);
}


//Writes one register value to LIS35
void LIS35_I2C_WriteRegister(unsigned char addr, unsigned char v)
{
  HAL_I2C_Mem_Write(&hi2c_LIS35, LIS35_I2C_Addr, addr, 1, &v, 1, 50);
}


//Gets XYZ values from LIS35
void LIS35_I2C_GetXYZacc(signed char * x, signed char * y, signed char * z)
{
  //Reads 5 registers values into buffer in multiple bytes mode, buffer[0]=x, [2]=y, [3]=z
  unsigned char Buffer[5]={0};

  //Read 5 consecutive registers in autoincrement mode starting from X axis
  HAL_I2C_Mem_Read(&hi2c_LIS35, LIS35_I2C_Addr, LIS35_REG_OUTX|LIS35_ADDR_INC_I2C, 1, Buffer, 5, 50);
  //Assign data to axis variables
  *x=(signed char)Buffer[0];
  *y=(signed char)Buffer[2];
  *z=(signed char)Buffer[4];

}
