//******************************************************************************
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    AUTHOR SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//******************************************************************************

#define HAL_SPI_MODULE_ENABLED
#define HAL_GPIO_MODULE_ENABLED
#include "mems2_spi_lib.h"
#include "stm32f1xx_hal.h"


SPI_HandleTypeDef hspi_LIS35;

//LIS35 hardware initialization for SPI operation
void LIS35_SPI_HwConfig(void)
{ //GPIO and SPI configuration

  GPIO_InitTypeDef GPIO_InitStruct;

  if (LIS35_SPI == SPI1) __HAL_RCC_SPI1_CLK_ENABLE(); else
  if (LIS35_SPI == SPI2) __HAL_RCC_SPI2_CLK_ENABLE();

  if (LIS35_SPI_GPIO == GPIOA) __HAL_RCC_GPIOA_CLK_ENABLE(); else
  if (LIS35_SPI_GPIO == GPIOB) __HAL_RCC_GPIOB_CLK_ENABLE(); else
  if (LIS35_SPI_GPIO == GPIOC) __HAL_RCC_GPIOC_CLK_ENABLE(); else
  if (LIS35_SPI_GPIO == GPIOD) __HAL_RCC_GPIOD_CLK_ENABLE(); else
  if (LIS35_SPI_GPIO == GPIOE) __HAL_RCC_GPIOE_CLK_ENABLE();

  //GPIO Configuration
  GPIO_InitStruct.Pin = LIS35_SS;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(LIS35_SPI_GPIO, &GPIO_InitStruct);
  HAL_GPIO_WritePin(LIS35_SPI_GPIO, LIS35_SS, GPIO_PIN_SET); //Set NSS state high

  GPIO_InitStruct.Pin = LIS35_SCK|LIS35_MOSI;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(LIS35_SPI_GPIO, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = LIS35_MISO;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(LIS35_SPI_GPIO, &GPIO_InitStruct);


  //SPI Config
  hspi_LIS35.Instance = LIS35_SPI;
  hspi_LIS35.Init.Mode = SPI_MODE_MASTER;
  hspi_LIS35.Init.Direction = SPI_DIRECTION_2LINES;
  hspi_LIS35.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi_LIS35.Init.CLKPolarity = SPI_POLARITY_HIGH;
  hspi_LIS35.Init.CLKPhase = SPI_PHASE_2EDGE;
  hspi_LIS35.Init.NSS = SPI_NSS_SOFT;
  hspi_LIS35.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_256;
  hspi_LIS35.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi_LIS35.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi_LIS35.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi_LIS35.Init.CRCPolynomial = 10;
  HAL_SPI_Init(&hspi_LIS35);
}


//Set NSS line low, activate slave
void SPI_NSS_Enable(void)
{
	HAL_GPIO_WritePin(LIS35_SPI_GPIO, LIS35_SS, GPIO_PIN_RESET);
}


//Set NSS line high, deactivate slave
void SPI_NSS_Disable(void)
{
	HAL_GPIO_WritePin(LIS35_SPI_GPIO, LIS35_SS, GPIO_PIN_SET);
}


//LIS35 initialization
//Range - select measurement range 2 = +-2g; 9=+-9g
char LIS35_SPI_Init(unsigned char Range)
{
  unsigned char RegVal, LIS35Settings;
  
  if (Range!=2 && Range!=9) return LIS35_ERROR;                             //Check Range selection
  //Reset LIS35 configuration
  LIS35_SPI_WriteRegister(LIS35_WRITE|LIS35_ADDR_NO_INC|LIS35_REG_CR2, LIS35_REG_CR2_BOOT);

  //Write settings - activate all axis
  LIS35Settings = LIS35_REG_CR1_DR_100HZ | LIS35_REG_CR1_XEN | LIS35_REG_CR1_YEN | LIS35_REG_CR1_ZEN | LIS35_REG_CR1_ACTIVE;
  if (Range==9) {LIS35Settings = LIS35Settings | LIS35_REG_CR1_FULL_SCALE;} //Full Scale select
  LIS35_SPI_WriteRegister(LIS35_REG_CR1, LIS35Settings);
  //Read configuration - if OK, LIS35 is up and running
  LIS35_SPI_ReadRegister(LIS35_REG_CR1, &RegVal);
  if (RegVal == LIS35Settings) return LIS35_OK;
  else return LIS35_ERROR;
}


//Reads one register value from LIS35
void LIS35_SPI_ReadRegister(unsigned char addr, unsigned char *v)
{
	unsigned char DataReceived[2];
	unsigned char DataToTransmit[2];

	DataToTransmit[0]=LIS35_READ|LIS35_ADDR_NO_INC|addr;
	DataToTransmit[1]=0xFF;

	SPI_NSS_Enable();
	HAL_SPI_TransmitReceive(&hspi_LIS35, DataToTransmit, DataReceived, 2, 50);
	SPI_NSS_Disable();
	*v=DataReceived[1];
}


//Writes one register value to LIS35
void LIS35_SPI_WriteRegister(unsigned char addr, unsigned char v)
{
	unsigned char DataReceived[2];
	unsigned char DataToTransmit[2];

	DataToTransmit[0]=LIS35_WRITE|LIS35_ADDR_NO_INC|addr;
	DataToTransmit[1]=v;

	SPI_NSS_Enable();
	HAL_SPI_TransmitReceive(&hspi_LIS35, DataToTransmit, DataReceived, 2, 50);
	SPI_NSS_Disable();
}



//Gets XYZ values from LIS35
void LIS35_SPI_GetPosition(signed char * x, signed char * y, signed char * z)
{
	unsigned char DataReceived[6];
	unsigned char DataToTransmit[6]={0};

	DataToTransmit[0]=LIS35_READ|LIS35_ADDR_INC_SPI|LIS35_REG_OUTX;
	SPI_NSS_Enable();
    HAL_SPI_TransmitReceive(&hspi_LIS35, DataToTransmit, DataReceived, 6, 50);
	SPI_NSS_Disable();

	*x=DataReceived[1];
	*y=DataReceived[3];
	*z=DataReceived[5];
}
