//******************************************************************************
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    AUTHOR SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//******************************************************************************

#define HAL_SPI_MODULE_ENABLED
#define HAL_GPIO_MODULE_ENABLED
#include "ssd1331_spi_lib.h"
#include "stm32f1xx_hal.h"


SPI_HandleTypeDef hspi_SSD1331;

//SSD1331 hardware initialization for SPI operation
void SSD1331_SPI_HwConfig()
{

  GPIO_InitTypeDef  GPIO_InitStruct;

  //clocks activation
  if (SSD1331_SPI == SPI1) __HAL_RCC_SPI1_CLK_ENABLE(); else
  if (SSD1331_SPI == SPI2) __HAL_RCC_SPI2_CLK_ENABLE();

  if (SSD1331_SPI_GPIO == GPIOA) __HAL_RCC_GPIOA_CLK_ENABLE(); else
  if (SSD1331_SPI_GPIO == GPIOB) __HAL_RCC_GPIOB_CLK_ENABLE(); else
  if (SSD1331_SPI_GPIO == GPIOC) __HAL_RCC_GPIOC_CLK_ENABLE(); else
  if (SSD1331_SPI_GPIO == GPIOD) __HAL_RCC_GPIOD_CLK_ENABLE(); else
  if (SSD1331_SPI_GPIO == GPIOE) __HAL_RCC_GPIOE_CLK_ENABLE();

  //GPIO Config
  GPIO_InitStruct.Pin = SSD1331_SS|SSD1331_DC|SSD1331_RES;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(SSD1331_SPI_GPIO, &GPIO_InitStruct);
  HAL_GPIO_WritePin(SSD1331_SPI_GPIO, SSD1331_SS|SSD1331_DC|SSD1331_RES, GPIO_PIN_SET); //Set NSS, DC, RES  state high

  GPIO_InitStruct.Pin = SSD1331_SCK|SSD1331_MOSI;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(SSD1331_SPI_GPIO, &GPIO_InitStruct);

  //SPI Config
  hspi_SSD1331.Instance = SSD1331_SPI;
  hspi_SSD1331.Init.Mode = SPI_MODE_MASTER;
  hspi_SSD1331.Init.Direction = SPI_DIRECTION_2LINES;
  hspi_SSD1331.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi_SSD1331.Init.CLKPolarity = SPI_POLARITY_HIGH;
  hspi_SSD1331.Init.CLKPhase = SPI_PHASE_2EDGE;
  hspi_SSD1331.Init.NSS = SPI_NSS_SOFT;
  hspi_SSD1331.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_8;
  hspi_SSD1331.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi_SSD1331.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi_SSD1331.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi_SSD1331.Init.CRCPolynomial = 10;
  HAL_SPI_Init(&hspi_SSD1331);
}


//SSD1331 initialization
void SSD1331_SPI_Init()
{
  SSD1331_SPI_WriteByte(SSD1331_DISPLAY_OFF, SSD1331_CMD);            //Display Off
  SSD1331_SPI_WriteByte(SSD1331_SET_CONTRAST_A, SSD1331_CMD);         //Set contrast for color A
  SSD1331_SPI_WriteByte(0xFF, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_CONTRAST_B, SSD1331_CMD);         //Set contrast for color B
  SSD1331_SPI_WriteByte(0xFF, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_CONTRAST_C, SSD1331_CMD);         //Set contrast for color C
  SSD1331_SPI_WriteByte(0xFF, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_MASTER_CURRENT_CONTROL, SSD1331_CMD); //Master current control
  SSD1331_SPI_WriteByte(0x07, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_PRECHARGE_SPEED_A, SSD1331_CMD);  //Set Second Pre-change Speed For ColorA
  SSD1331_SPI_WriteByte(0x64, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_PRECHARGE_SPEED_B, SSD1331_CMD);  //Set Second Pre-change Speed For ColorB
  SSD1331_SPI_WriteByte(0x78, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_PRECHARGE_SPEED_C, SSD1331_CMD);  //Set Second Pre-change Speed For ColorC
  SSD1331_SPI_WriteByte(0x64, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_REMAP, SSD1331_CMD);              //Set remap & data format
  SSD1331_SPI_WriteByte(0x72, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_DISPLAY_START_LINE, SSD1331_CMD); //Set display Start Line
  SSD1331_SPI_WriteByte(0x0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_DISPLAY_OFFSET, SSD1331_CMD);     //Set display offset
  SSD1331_SPI_WriteByte(0x0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_NORMAL_DISPLAY, SSD1331_CMD);         //Set display mode
  SSD1331_SPI_WriteByte(SSD1331_SET_MULTIPLEX_RATIO, SSD1331_CMD);    //Set multiplex ratio
  SSD1331_SPI_WriteByte(0x3F, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_MASTER_CONFIG, SSD1331_CMD);      //Set master configuration
  SSD1331_SPI_WriteByte(0x8E, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_POWER_SAVE_MODE, SSD1331_CMD);        //Set Power Save Mode
  SSD1331_SPI_WriteByte(0x00, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_PHASE_PERIOD_ADJUSTMENT, SSD1331_CMD);//Phase 1 and 2 period adjustment
  SSD1331_SPI_WriteByte(0x31, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_DISPLAY_CLOCK_DIV, SSD1331_CMD);      //Oscillator frequency & display clock divider
  SSD1331_SPI_WriteByte(0xF0, SSD1331_CMD);                           //890kHz/1
  SSD1331_SPI_WriteByte(SSD1331_SET_PRECHARGE_VOLTAGE, SSD1331_CMD);  //Set Pre-Change Level
  SSD1331_SPI_WriteByte(0x3A, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_SET_V_VOLTAGE, SSD1331_CMD);          //Set Vcomh
  SSD1331_SPI_WriteByte(0x3E, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_DEACTIVE_SCROLLING, SSD1331_CMD);     //Disable scrolling
  SSD1331_SPI_WriteByte(SSD1331_DIM_MODE_SETTING, SSD1331_CMD);       //Set dim mode parameters
  SSD1331_SPI_WriteByte(0x00, SSD1331_CMD);                           //reserved, must be 0
  SSD1331_SPI_WriteByte(0x40, SSD1331_CMD);                           //R contrast
  SSD1331_SPI_WriteByte(0x40, SSD1331_CMD);                           //G contrast
  SSD1331_SPI_WriteByte(0x40, SSD1331_CMD);                           //B contrast
  SSD1331_SPI_WriteByte(0x1E, SSD1331_CMD);                           //pre-charge

  SSD1331_SPI_WriteByte(SSD1331_DISPLAY_ON_NORMAL, SSD1331_CMD);      //Set display on
  SSD1331_ClearScreen();
}


//Write one byte to SSD1331 as Command (0) or Data (1)
void SSD1331_SPI_WriteByte(unsigned char v, unsigned char DC)
{
  if (DC==SSD1331_CMD)    //Command
	HAL_GPIO_WritePin(SSD1331_SPI_GPIO, SSD1331_DC, GPIO_PIN_RESET);
  else                    //Data
	HAL_GPIO_WritePin(SSD1331_SPI_GPIO, SSD1331_DC, GPIO_PIN_SET);

  HAL_GPIO_WritePin(SSD1331_SPI_GPIO, SSD1331_SS, GPIO_PIN_RESET);   //NSS enable
  HAL_SPI_Transmit(&hspi_SSD1331, &v, 1, 50);
  HAL_GPIO_WritePin(SSD1331_SPI_GPIO, SSD1331_SS, GPIO_PIN_SET);     //NSS diable
  HAL_GPIO_WritePin(SSD1331_SPI_GPIO, SSD1331_DC, GPIO_PIN_SET);
}


//Clear Screen (using hardware built-in function)
void SSD1331_ClearScreen()
{
  //SSD1331_Fill_Rect(0,0,95,63,color,color);
  SSD1331_SPI_WriteByte(SSD1331_CLEAR_WINDOW, SSD1331_CMD);
  SSD1331_SPI_WriteByte(0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(95, SSD1331_CMD);
  SSD1331_SPI_WriteByte(63, SSD1331_CMD);
  HAL_Delay(1); //delay for allowing driver to finish clearing the whole display
}


//Draw a pixel in x,y position with a given color
void SSD1331_DrawPixel(unsigned char x, unsigned char y, unsigned short int color)
{
  if (x >= SSD1331_OLED_WIDTH || y >= SSD1331_OLED_HEIGHT) {
	return;
  }

  SSD1331_SPI_WriteByte(SSD1331_SET_COLUMN_ADDRESS, SSD1331_CMD);       //set column
  SSD1331_SPI_WriteByte(x, SSD1331_CMD);
  SSD1331_SPI_WriteByte(x, SSD1331_CMD);

  SSD1331_SPI_WriteByte(SSD1331_SET_ROW_ADDRESS, SSD1331_CMD);          //set row
  SSD1331_SPI_WriteByte(y, SSD1331_CMD);
  SSD1331_SPI_WriteByte(y, SSD1331_CMD);

  SSD1331_SPI_WriteByte(color >> 8, SSD1331_DATA);                      //send 16bit color, high byte
  SSD1331_SPI_WriteByte(color, SSD1331_DATA);                           //low byte
}


//Draw filled-in rectangle between x0,y0 and x1,y1 (use built-in hardware function) with given fill-in and outline colors
void SSD1331_FillRect(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, unsigned short int fillColor, unsigned short int lineColor) {
  SSD1331_SPI_WriteByte(SSD1331_FILL_EN, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_ENABLE_FILL, SSD1331_CMD);
  SSD1331_SPI_WriteByte(SSD1331_DRAW_RECTANGLE, SSD1331_CMD);
  SSD1331_SPI_WriteByte(x0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(y0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(x1, SSD1331_CMD);
  SSD1331_SPI_WriteByte(y1, SSD1331_CMD);
  SSD1331_SPI_WriteByte((((lineColor&0xF800)>>11)<<1), SSD1331_CMD);
  SSD1331_SPI_WriteByte((((lineColor&0x07E0)>>5 )   ), SSD1331_CMD);
  SSD1331_SPI_WriteByte((((lineColor&0x001F)    )<<1), SSD1331_CMD);
  SSD1331_SPI_WriteByte((((fillColor&0xF800)>>11)<<1), SSD1331_CMD);
  SSD1331_SPI_WriteByte((((fillColor&0x07E0)>>5 )   ), SSD1331_CMD);
  SSD1331_SPI_WriteByte((((fillColor&0x001F)    )<<1), SSD1331_CMD);
  HAL_Delay(1);
}


//Draw line from x0,y0 to x1, y1 (use built-in hardware function)
void SSD1331_DrawLine(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, unsigned short int lineColor)
{
  SSD1331_SPI_WriteByte(SSD1331_DRAW_LINE, SSD1331_CMD);
  SSD1331_SPI_WriteByte(x0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(y0, SSD1331_CMD);
  SSD1331_SPI_WriteByte(x1, SSD1331_CMD);
  SSD1331_SPI_WriteByte(y1, SSD1331_CMD);
  SSD1331_SPI_WriteByte((((lineColor&0xF800)>>11)<<1), SSD1331_CMD);
  SSD1331_SPI_WriteByte((((lineColor&0x07E0)>>5 )   ), SSD1331_CMD);
  SSD1331_SPI_WriteByte((((lineColor&0x001F)    )<<1), SSD1331_CMD);
}


//Draw circle with a given color, center in x0,y0 and a given radius
void  SSD1331_DrawCircle(int x0, int y0, int radius, unsigned int color) {
  int f = 1 - radius;
  int ddF_x = 0;
  int ddF_y = -2 * radius;
  int x = 0;
  int y = radius;

  SSD1331_DrawPixel(x0, y0 + radius, color);     //s1
  SSD1331_DrawPixel(x0, y0 - radius, color);     //s2
  SSD1331_DrawPixel(x0 + radius, y0, color);     //s3
  SSD1331_DrawPixel(x0 - radius, y0, color);     //s4
  while (x < y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x + 1;
    SSD1331_DrawPixel(x0 + x, y0 + y, color);     //1
    SSD1331_DrawPixel(x0 - x, y0 + y, color);     //2
    SSD1331_DrawPixel(x0 + x, y0 - y, color);     //3
    SSD1331_DrawPixel(x0 - x, y0 - y, color);     //4
    SSD1331_DrawPixel(x0 + y, y0 + x, color);     //5
    SSD1331_DrawPixel(x0 - y, y0 + x, color);     //6
    SSD1331_DrawPixel(x0 + y, y0 - x, color);     //7
    SSD1331_DrawPixel(x0 - y, y0 - x, color);     //8
  }
}


//Draw circle filled with a given color, circle center in x0, y0 and a given radius
void SSD1331_FillCircle(int x0, int y0, int radius, unsigned int color) {
  int f = 1 - radius;
  int ddF_x = 0;
  int ddF_y = -2 * radius;
  int x = 0;
  int y = radius;

  SSD1331_DrawLine(x0, y0 + radius, x0, y0 - radius, color);  //s1-2
  SSD1331_DrawPixel(x0 + radius, y0, color);                  //s3
  SSD1331_DrawPixel(x0 - radius, y0, color);                  //s4
  while (x < y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x + 1;

    SSD1331_DrawLine(x0 + x, y0 + y, x0 + x, y0 - y, color);  //1-3
    SSD1331_DrawLine(x0 - x, y0 + y, x0 - x, y0 - y, color);  //2-4
    SSD1331_DrawLine(x0 + y, y0 + x, x0 + y, y0 - x, color);  //5-7
    SSD1331_DrawLine(x0 - y, y0 + x, x0 - y, y0 - x, color);  //6-8
  }
}


//Write text starting from X, Y position, set font and background color, use fontData as character pattern
void SSD1331_WriteTextXY(const char *text, int xs, int ys, unsigned int fontColor, unsigned int bkgColor, const unsigned char * FontData) {
  //Write text at x,y position with a given foreground and background color
  unsigned int i, j, x, y;
  unsigned char pixelRow;
  unsigned char Mask=0x80;
  unsigned char charCode;

  x=xs;
  y=ys;
    while(*text!=0){
      if (x+FontData[0]>95){         //if next character is out of display, move to the next line
        y+=FontData[1];
        x=xs;
      }

      SSD1331_SPI_WriteByte(SSD1331_SET_COLUMN_ADDRESS, SSD1331_CMD);
      SSD1331_SPI_WriteByte(x, SSD1331_CMD);
      SSD1331_SPI_WriteByte(x+FontData[0]-1, SSD1331_CMD);

      SSD1331_SPI_WriteByte(SSD1331_SET_ROW_ADDRESS, SSD1331_CMD);
      SSD1331_SPI_WriteByte(y, SSD1331_CMD);
      SSD1331_SPI_WriteByte(y+FontData[1]-1, SSD1331_CMD);

      // Write char data
      charCode=*text-0x1F;
      for(i = 0; i <FontData[1]; i++) {
        pixelRow=FontData[(charCode)*FontData[1]+i];    //get one row of char pattern
        Mask = 0x80;
        for (j=0; j<FontData[0]; j++){
          if ((pixelRow & Mask) == 0){                  //put pixel by pixel and set its color
              SSD1331_SPI_WriteByte(((bkgColor>>8)&0x00FF), SSD1331_DATA);
              SSD1331_SPI_WriteByte(((bkgColor   )&0x00FF), SSD1331_DATA);
          } else {
              SSD1331_SPI_WriteByte(((fontColor>>8)&0x00FF), SSD1331_DATA);
              SSD1331_SPI_WriteByte(((fontColor   )&0x00FF), SSD1331_DATA);
          }
          Mask = Mask >> 1;
        }
      }
      text++;
      x+=FontData[0];
    }
}


//writes bitmap image stored in 565 format (1 pixel made of: 1st byte = RRRRRGGG & 2nd byte = GGGBBBB)
void SSD1331_Write565Bitmap(const unsigned char *buffer, int xPos, int yPos, int xSize, int ySize) {
  unsigned long int i;

  SSD1331_SPI_WriteByte(SSD1331_SET_COLUMN_ADDRESS, SSD1331_CMD);
  SSD1331_SPI_WriteByte(xPos, SSD1331_CMD);
  SSD1331_SPI_WriteByte(xPos+xSize-1, SSD1331_CMD);

  SSD1331_SPI_WriteByte(SSD1331_SET_ROW_ADDRESS, SSD1331_CMD);
  SSD1331_SPI_WriteByte(yPos,         SSD1331_CMD);
  SSD1331_SPI_WriteByte(yPos+ySize-1, SSD1331_CMD);

  for(i = 0; i <xSize*ySize*2; i+=2){
    SSD1331_SPI_WriteByte(((buffer[i  ]>>8)&0x00FF), SSD1331_DATA);
    SSD1331_SPI_WriteByte(((buffer[i+1]   )&0x00FF), SSD1331_DATA);
  }

  HAL_Delay(1);
}


